import type { LookupByPath } from '@rushstack/rush-sdk';
import type { IResolverCacheFile } from '@rushstack/webpack-workspace-resolve-plugin';
import type { PnpmShrinkwrapFile } from './externals';
import type { IResolverContext } from './types';
/**
 * The only parts of a RushConfigurationProject needed by this tool.
 * Reduced for unit test typings.
 */
export interface IPartialRushProject {
    projectFolder: string;
    packageJson: {
        name: string;
    };
}
export interface IPlatformInfo {
    os: typeof process.platform;
    cpu: typeof process.arch;
    libc: 'glibc' | 'musl';
}
/**
 * Options for computing the resolver cache from a lockfile.
 */
export interface IComputeResolverCacheFromLockfileOptions {
    /**
     * The root folder of the workspace being installed
     */
    workspaceRoot: string;
    /**
     * The common root path to trim from the description file roots for brevity
     */
    commonPrefixToTrim: string;
    /**
     * Information about the platform Rush is running on
     */
    platformInfo: IPlatformInfo;
    /**
     * A lookup of projects by their importer path
     */
    projectByImporterPath: Pick<LookupByPath<IPartialRushProject>, 'findChildPath'>;
    /**
     * The lockfile to compute the cache from
     */
    lockfile: PnpmShrinkwrapFile;
    /**
     * A callback to process external packages after they have been enumerated.
     * Broken out as a separate function to facilitate testing without hitting the disk.
     * @remarks This is useful for fetching additional data from the pnpm store
     * @param contexts - The current context information per description file root
     * @param missingOptionalDependencies - The set of optional dependencies that were not installed
     * @returns A promise that resolves when the external packages have been processed
     */
    afterExternalPackagesAsync?: (contexts: Map<string, IResolverContext>, missingOptionalDependencies: Set<string>) => Promise<void>;
}
/**
 * Given a lockfile and information about the workspace and platform, computes the resolver cache file.
 * @param params - The options for computing the resolver cache
 * @returns A promise that resolves with the resolver cache file
 */
export declare function computeResolverCacheFromLockfileAsync(params: IComputeResolverCacheFromLockfileOptions): Promise<IResolverCacheFile>;
//# sourceMappingURL=computeResolverCacheFromLockfileAsync.d.ts.map