"use strict";
// Copyright (c) Microsoft Corporation. All rights reserved. Licensed under the MIT license.
// See LICENSE in the project root for license information.
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createContextSerializer = exports.depPathToFilenameUnescaped = exports.resolveDependencies = exports.getDescriptionFileRootFromKey = exports.resolveDependencyKey = exports.depPathToFilename = exports.createBase32Hash = void 0;
const node_crypto_1 = require("node:crypto");
const path = __importStar(require("node:path"));
const MAX_LENGTH_WITHOUT_HASH = 120 - 26 - 1;
const BASE32 = 'abcdefghijklmnopqrstuvwxyz234567'.split('');
// https://github.com/swansontec/rfc4648.js/blob/ead9c9b4b68e5d4a529f32925da02c02984e772c/src/codec.ts#L82-L118
function createBase32Hash(input) {
    const data = (0, node_crypto_1.createHash)('md5').update(input).digest();
    const mask = 0x1f;
    let out = '';
    let bits = 0; // Number of bits currently in the buffer
    let buffer = 0; // Bits waiting to be written out, MSB first
    for (let i = 0; i < data.length; ++i) {
        // eslint-disable-next-line no-bitwise
        buffer = (buffer << 8) | (0xff & data[i]);
        bits += 8;
        // Write out as much as we can:
        while (bits > 5) {
            bits -= 5;
            // eslint-disable-next-line no-bitwise
            out += BASE32[mask & (buffer >> bits)];
        }
    }
    // Partial character:
    if (bits) {
        // eslint-disable-next-line no-bitwise
        out += BASE32[mask & (buffer << (5 - bits))];
    }
    return out;
}
exports.createBase32Hash = createBase32Hash;
// https://github.com/pnpm/pnpm/blob/f394cfccda7bc519ceee8c33fc9b68a0f4235532/packages/dependency-path/src/index.ts#L167-L189
function depPathToFilename(depPath) {
    let filename = depPathToFilenameUnescaped(depPath).replace(/[\\/:*?"<>|]/g, '+');
    if (filename.includes('(')) {
        filename = filename.replace(/(\)\()|\(/g, '_').replace(/\)$/, '');
    }
    if (filename.length > 120 || (filename !== filename.toLowerCase() && !filename.startsWith('file+'))) {
        return `${filename.substring(0, MAX_LENGTH_WITHOUT_HASH)}_${createBase32Hash(filename)}`;
    }
    return filename;
}
exports.depPathToFilename = depPathToFilename;
/**
 * Computes the root folder for a dependency from a reference to it in another package
 * @param lockfileFolder - The folder that contains the lockfile
 * @param key - The key of the dependency
 * @param specifier - The specifier in the lockfile for the dependency
 * @param context - The owning package
 * @returns The identifier for the dependency
 */
function resolveDependencyKey(lockfileFolder, key, specifier, context) {
    if (specifier.startsWith('/')) {
        return getDescriptionFileRootFromKey(lockfileFolder, specifier);
    }
    else if (specifier.startsWith('link:')) {
        if (context.isProject) {
            return path.posix.join(context.descriptionFileRoot, specifier.slice(5));
        }
        else {
            return path.posix.join(lockfileFolder, specifier.slice(5));
        }
    }
    else if (specifier.startsWith('file:')) {
        return getDescriptionFileRootFromKey(lockfileFolder, specifier, key);
    }
    else {
        return getDescriptionFileRootFromKey(lockfileFolder, `/${key}@${specifier}`);
    }
}
exports.resolveDependencyKey = resolveDependencyKey;
/**
 * Computes the physical path to a dependency based on its entry
 * @param lockfileFolder - The folder that contains the lockfile during installation
 * @param key - The key of the dependency
 * @param name - The name of the dependency, if provided
 * @returns The physical path to the dependency
 */
function getDescriptionFileRootFromKey(lockfileFolder, key, name) {
    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
    if (!key.startsWith('file:')) {
        name = key.slice(1, key.indexOf('@', 2));
    }
    if (!name) {
        throw new Error(`Missing package name for ${key}`);
    }
    const originFolder = `${lockfileFolder}/node_modules/.pnpm/${depPathToFilename(key)}/node_modules`;
    const descriptionFileRoot = `${originFolder}/${name}`;
    return descriptionFileRoot;
}
exports.getDescriptionFileRootFromKey = getDescriptionFileRootFromKey;
function resolveDependencies(lockfileFolder, collection, context) {
    for (const [key, value] of Object.entries(collection)) {
        const version = typeof value === 'string' ? value : value.version;
        const resolved = resolveDependencyKey(lockfileFolder, key, version, context);
        context.deps.set(key, resolved);
    }
}
exports.resolveDependencies = resolveDependencies;
/**
 *
 * @param depPath - The path to the dependency
 * @returns The folder name for the dependency
 */
function depPathToFilenameUnescaped(depPath) {
    if (depPath.indexOf('file:') !== 0) {
        if (depPath.startsWith('/')) {
            depPath = depPath.slice(1);
        }
        return depPath;
    }
    return depPath.replace(':', '+');
}
exports.depPathToFilenameUnescaped = depPathToFilenameUnescaped;
/**
 *
 * @param missingOptionalDependencies - The set of optional dependencies that were not installed
 * @param contexts - The map of context roots to their respective contexts
 * @param commonPathPrefix - The common root path to trim
 * @returns A function that serializes a context into a format that can be written to disk
 */
function createContextSerializer(missingOptionalDependencies, contexts, commonPathPrefix) {
    return ([descriptionFileRoot, context]) => {
        var _a;
        const { deps } = context;
        let hasAnyDeps = false;
        const serializedDeps = {};
        for (const [key, contextRoot] of deps) {
            if (missingOptionalDependencies.has(contextRoot)) {
                continue;
            }
            const resolutionContext = contexts.get(contextRoot);
            if (!resolutionContext) {
                throw new Error(`Missing context for ${contextRoot}!`);
            }
            serializedDeps[key] = resolutionContext.ordinal;
            hasAnyDeps = true;
        }
        if (!context.name) {
            throw new Error(`Missing name for ${descriptionFileRoot}`);
        }
        const serializedContext = {
            name: context.name,
            root: descriptionFileRoot.slice(commonPathPrefix.length),
            dirInfoFiles: ((_a = context.nestedPackageDirs) === null || _a === void 0 ? void 0 : _a.length) ? context.nestedPackageDirs : undefined,
            deps: hasAnyDeps ? serializedDeps : undefined
        };
        return serializedContext;
    };
}
exports.createContextSerializer = createContextSerializer;
//# sourceMappingURL=helpers.js.map